@@ -9,7 +9,7 @@
 //computes the position of a takens' vector that falls into the (a,b)
 //unwrapped box in the wrapped 2d box grid formed with "numberBoxes" in 
 //each dimension
-inline int position(int a, int b,int numberBoxes){
+static inline int position(int a, int b,int numberBoxes){
   //a specifies the file, b the colum in the imaginary unwrapped 2D cover with boxes
   return ( 
              numberBoxes * ( MOD(a+OFFSET,numberBoxes) )
